program ShowWindows;

{$mode objfpc}{$H+}

uses
  Classes,
  SysUtils,
  Exec, Intuition, agraphics,
  fpg_base, fpg_main, fpg_form, fpg_tree, fpg_checkbox, fpg_button,
  fpg_imagelist, fpg_label, fpg_dialogs, fpg_combobox, fpg_utils,
  fpg_grid;

const
  ENTRYTYPE_SCREEN = 0;
  ENTRYTYPE_WINDOW = 1;

type

  TScrWinEntry = record
    Address: Pointer;
    PtrType: Integer;
  end;

  TMainForm = class(TfpgForm)
  private
    {@VFD_HEAD_BEGIN: MainForm}
    tree: TfpgTreeView;
    btnRefresh: TfpgButton;
    btnCollapseAll: TfpgButton;
    btnExpandAll: TfpgButton;
    Grid1: TfpgStringGrid;
    btnMarkWindow: TfpgButton;
    {@VFD_HEAD_END: MainForm}
    IBase: PIntuitionBase;
    ItemList: array of TScrWinEntry;
    procedure btnRefreshClicked(Sender: TObject);
    procedure TreeNodeChanged(Sender: TObject);
    procedure btnCollapseAllClicked(Sender: TObject);
    procedure btnExpandAllClicked(Sender: TObject);
    procedure btnShowSelected(Sender: TObject);
    procedure RefreshWindowList;
    function StillExists(Idx: Integer): Pointer;
    procedure InspectScreen(Scr: PScreen);
    procedure InspectWindow(Win: PWindow);
    procedure AddCell(N1, N2: string); overload;
    procedure AddCell(N1: string; Pt: Pointer); overload;
    procedure MarkWindow(Window: PWindow);

  public
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;
    procedure   AfterCreate; override;
  end;

{ TMainForm }

procedure TMainForm.RefreshWindowList;

  function AddToList(Ptr: Pointer; PtrType: Integer): Integer;
  begin
    Result := Length(ItemList);
    SetLength(ItemList, Result + 1);
    ItemList[Result].Address := Ptr;
    ItemList[Result].PtrType := PtrType;
  end;

  procedure CheckChilds(Window: PWindow; Node: TfpgTreeNode);
  var
    NextNode: TfpgTreeNode;
  begin
    if not Assigned(Window) then
      Exit;
    NextNode := Node.AppendText('Window $' + IntToHex(PtrUInt(Window), SizeOf(pointer)) + ' ' + string(Window^.title));
    NextNode.Data := Pointer(AddToList(Window, ENTRYTYPE_WINDOW));
    CheckChilds(Window^.FirstChild, NextNode);
    CheckChilds(Window^.NextChild, Node);
  end;

  procedure CheckWindows(Window: PWindow; Node: TfpgTreeNode);
  var
    NextNode: TfpgTreeNode;
  begin
    if not Assigned(Window) then
      Exit;
    if not Assigned(Window^.Parent2) then
    begin
      NextNode := Node.AppendText('Window $' + IntToHex(PtrUInt(Window), SizeOf(pointer)) + ' ' + string(Window^.title));
      NextNode.Data := Pointer(AddToList(Window, ENTRYTYPE_WINDOW));
      CheckChilds(Window^.FirstChild, NextNode);
    end;
    CheckWindows(Window^.NextWindow, Node);
  end;

  procedure CheckScreens(Screen: PScreen);
  var
    WinNode: TfpgTreeNode;
  begin
    if not Assigned(Screen) then
      Exit;
    WinNode := tree.RootNode.AppendText('Screen $' + IntToHex(PtrUInt(Screen), SizeOf(pointer)) + ' ' + string(screen^.title));
    WinNode.Data := Pointer(AddToList(Screen, ENTRYTYPE_SCREEN));
    CheckWindows(Screen^.FirstWindow, WinNode);
    CheckScreens(Screen^.NextScreen);
  end;

begin
  Tree.BeginUpdate;
  Tree.RootNode.Clear;
  SetLength(ItemList, 0);
  if Assigned(IBase) then
    CheckScreens(IBase^.FirstScreen);
  Tree.EndUpdate;
  Tree.RootNode.Expand;
end;

function TMainForm.StillExists(Idx: Integer): Pointer;
var
  Scr: PScreen;
  Win: PWindow;
  Target: Pointer;
begin
  Result := nil;
  if (Idx >= 0) and (Idx < Length(ItemList)) then
  begin
    Target := ItemList[Idx].Address;
    if not Assigned(Target) then
      Exit;
    Scr := IBase^.FirstScreen;
    while Assigned(Scr) do
    begin
      if Pointer(Scr) = Target then
      begin
        Result := Pointer(Scr);
        Exit;
      end;
      Win := Scr^.FirstWindow;
      while Assigned(Win) do
      begin
        if Pointer(Win) = Target then
        begin
          Result := Pointer(Win);
          Exit;
        end;
        Win := Win^.NextWindow;
      end;
      Scr := Scr^.NextScreen;
    end;
  end;
end;

procedure TMainForm.btnRefreshClicked(Sender: TObject);
begin
  RefreshWindowList;
end;

procedure TMainForm.AddCell(N1, N2: string);
begin
  Grid1.RowCount := Grid1.RowCount + 1;
  Grid1.Cells[0, Grid1.RowCount - 1] := N1;
  Grid1.Cells[1, Grid1.RowCount - 1] := N2;
end;

procedure TMainForm.AddCell(N1: string; Pt: Pointer);
begin
  if Assigned(Pt) then
    AddCell(N1, '$' + IntToHex(PtrUInt(Pt), SizeOf(Pointer)))
  else
    AddCell(N1, 'nil');
end;

procedure TMainForm.InspectScreen(Scr: PScreen);
var
  str: string;
begin
  AddCell('Type', 'Screen');
  AddCell('Adress', Pointer(Scr));
  AddCell('Pos; Size', '' + IntToStr(Scr^.LeftEdge) + ',' + IntToStr(Scr^.TopEdge) + '; ' + IntToStr(Scr^.Width) + 'x' + IntToStr(Scr^.Height));
  AddCell('Title', string(Scr^.Title));
  AddCell('DefaultTitle', string(Scr^.DefaultTitle));
  if (Scr^.Flags and WBENCHSCREEN_f) <> 0 then
    Str := 'WBENCHSCREEN'
  else
    Str := 'CUSTOMSCREEN';
  if (Scr^.Flags and SHOWTITLE_f) <> 0 then
    Str := str + '|SHOWTITLE';
  if (Scr^.Flags and BEEPING_f) <> 0 then
    Str := str + '|BEEPING';
  if (Scr^.Flags and CUSTOMBITMAP_f) <> 0 then
    Str := str + '|CUSTOMBITMAP';
  if (Scr^.Flags and SCREENBEHIND_f) <> 0 then
    Str := str + '|SCREENBEHIND';
  if (Scr^.Flags and SCREENQUIET_f) <> 0 then
    Str := str + '|SCREENQUIET';
  if (Scr^.Flags and SCREENHIRES) <> 0 then
    Str := str + '|SCREENHIRES';
  if (Scr^.Flags and PENSHARED) <> 0 then
    Str := str + '|PENSHARED';
  if (Scr^.Flags and AUTOSCROLL) <> 0 then
    Str := str + '|AUTOSCROLL';
  AddCell('Flags', '($' + IntToHex(Scr^.Flags, 8) + ')' + str);
  AddCell('Bar', 'Height:' + IntToStr(Scr^.BarHeight) + ' Border:' + IntToStr(Scr^.BarVBorder) + 'x' + IntToStr(Scr^.BarHBorder));
  AddCell('MenuBar', 'Border:' + IntToStr(Scr^.MenuVBorder) + 'x' + IntToStr(Scr^.MenuHBorder));
  AddCell('Window Border', 'left:' + IntToStr(Scr^.WBorLeft) + ' top:' + IntToStr(Scr^.WBorTop) + ' right:' + IntToStr(Scr^.WBorRight) + ' bottom:' + IntToStr(Scr^.WBorBottom));
  AddCell('Font', Scr^.Font^.ta_Name + ':' + IntToStr(Scr^.Font^.ta_YSize));

end;

procedure TMainForm.InspectWindow(Win: PWindow);
var
  str: string;
begin
  AddCell('Type', 'Window');
  AddCell('Address', Pointer(Win));
  AddCell('Pos; Size', '' + IntToStr(Win^.LeftEdge) + ',' + IntToStr(Win^.TopEdge) + '; ' + IntToStr(Win^.Width) + 'x' + IntToStr(Win^.Height));
  AddCell('Min/Max Size', IntToStr(Win^.MinWidth) + 'x' + IntToStr(Win^.MinHeight) + ' / ' + IntToStr(Win^.MaxWidth) + 'x' + IntToStr(Win^.MaxHeight));
  AddCell('Title', string(Win^.Title));
  Str := '';
  if (Win^.Flags and WFLG_SIZEGADGET) <> 0 then
    Str := str + '|SIZEGADGET';
  if (Win^.Flags and WFLG_DRAGBAR) <> 0 then
    Str := str + '|DRAGBAR';
  if (Win^.Flags and WFLG_DEPTHGADGET) <> 0 then
    Str := str + '|DEPTHGADGET';
  if (Win^.Flags and WFLG_CLOSEGADGET) <> 0 then
    Str := str + '|CLOSEGADGET';
  if (Win^.Flags and WFLG_SIZEBRIGHT) <> 0 then
    Str := str + '|SIZEBRIGHT';
  if (Win^.Flags and WFLG_SIZEBBOTTOM) <> 0 then
    Str := str + '|SIZEBBOTTOM';
  if (Win^.Flags and WFLG_REFRESHBITS) <> 0 then
    Str := str + '|REFRESHBITS';
  if (Win^.Flags and WFLG_SMART_REFRESH) <> 0 then
    Str := str + '|SMART_REFRESH';
  if (Win^.Flags and WFLG_SIMPLE_REFRESH) <> 0 then
    Str := str + '|SIMPLE_REFRESH';
  if (Win^.Flags and WFLG_SUPER_BITMAP) <> 0 then
    Str := str + '|SUPER_BITMAP';
  if (Win^.Flags and WFLG_OTHER_REFRESH) <> 0 then
    Str := str + '|OTHER_REFRESH';
  if (Win^.Flags and WFLG_BACKDROP) <> 0 then
    Str := str + '|BACKDROP';
  if (Win^.Flags and WFLG_REPORTMOUSE) <> 0 then
    Str := str + '|REPORTMOUSE';
  if (Win^.Flags and WFLG_GIMMEZEROZERO) <> 0 then
    Str := str + '|GIMMEZEROZERO';
  if (Win^.Flags and WFLG_BORDERLESS) <> 0 then
    Str := str + '|BORDERLESS';
  if (Win^.Flags and WFLG_ACTIVATE) <> 0 then
    Str := str + '|ACTIVATE';
  if (Win^.Flags and WFLG_NOCAREREFRESH) <> 0 then
    Str := str + '|NOCAREREFRESH';
  if (Win^.Flags and WFLG_NEWLOOKMENUS) <> 0 then
    Str := str + '|NEWLOOKMENUS';
  if (Win^.Flags and WFLG_WINDOWACTIVE) <> 0 then
    Str := str + '|WINDOWACTIVE';
  if (Win^.Flags and WFLG_INREQUEST) <> 0 then
    Str := str + '|INREQUEST';
  if (Win^.Flags and WFLG_WINDOWREFRESH) <> 0 then
    Str := str + '|WINDOWREFRESH';
  if (Win^.Flags and WFLG_WBENCHWINDOW) <> 0 then
    Str := str + '|WBENCHWINDOW';
  if (Win^.Flags and WFLG_WINDOWTICKED) <> 0 then
    Str := str + '|WINDOWTICKED';
  if (Win^.Flags and WFLG_VISITOR) <> 0 then
    Str := str + '|VISITOR';
  if (Win^.Flags and WFLG_ZOOMED) <> 0 then
    Str := str + '|ZOOMED';
  if (Win^.Flags and WFLG_HASZOOM) <> 0 then
    Str := str + '|HASZOOM';
  if Length(Str) > 0 then
    Delete(str, 1, 1);
  AddCell('Flags', '($' + IntToHex(Win^.Flags, 8) + ')'+ str);
  str := '';
  if (Win^.IDCMPFlags and IDCMP_SIZEVERIFY) <> 0 then
    Str := str + '|SIZEVERIFY';
  if (Win^.IDCMPFlags and IDCMP_NEWSIZE) <> 0 then
    Str := str + '|NEWSIZE';
  if (Win^.IDCMPFlags and IDCMP_REFRESHWINDOW) <> 0 then
    Str := str + '|REFRESHWINDOW';
  if (Win^.IDCMPFlags and IDCMP_MOUSEBUTTONS) <> 0 then
    Str := str + '|MOUSEBUTTONS';
  if (Win^.IDCMPFlags and IDCMP_MOUSEMOVE) <> 0 then
    Str := str + '|MOUSEMOVE';
  if (Win^.IDCMPFlags and IDCMP_GADGETDOWN) <> 0 then
    Str := str + '|GADGETDOWN';
  if (Win^.IDCMPFlags and IDCMP_GADGETUP) <> 0 then
    Str := str + '|GADGETUP';
  if (Win^.IDCMPFlags and IDCMP_REQSET) <> 0 then
    Str := str + '|REQSET';
  if (Win^.IDCMPFlags and IDCMP_MENUPICK) <> 0 then
    Str := str + '|MENUPICK';
  if (Win^.IDCMPFlags and IDCMP_CLOSEWINDOW) <> 0 then
    Str := str + '|CLOSEWINDOW';
  if (Win^.IDCMPFlags and IDCMP_RAWKEY) <> 0 then
    Str := str + '|RAWKEY';
  if (Win^.IDCMPFlags and IDCMP_REQVERIFY) <> 0 then
    Str := str + '|REQVERIFY';
  if (Win^.IDCMPFlags and IDCMP_REQCLEAR) <> 0 then
    Str := str + '|REQCLEAR';
  if (Win^.IDCMPFlags and IDCMP_MENUVERIFY) <> 0 then
    Str := str + '|MENUVERIFY';
  if (Win^.IDCMPFlags and IDCMP_NEWPREFS) <> 0 then
    Str := str + '|NEWPREFS';
  if (Win^.IDCMPFlags and IDCMP_DISKINSERTED) <> 0 then
    Str := str + '|DISKINSERTED';
  if (Win^.IDCMPFlags and IDCMP_DISKREMOVED) <> 0 then
    Str := str + '|DISKREMOVED';
  if (Win^.IDCMPFlags and IDCMP_WBENCHMESSAGE) <> 0 then
    Str := str + '|WBENCHMESSAGE';
  if (Win^.IDCMPFlags and IDCMP_ACTIVEWINDOW) <> 0 then
    Str := str + '|ACTIVEWINDOW';
  if (Win^.IDCMPFlags and IDCMP_INACTIVEWINDOW) <> 0 then
    Str := str + '|INACTIVEWINDOW';
  if (Win^.IDCMPFlags and IDCMP_DELTAMOVE) <> 0 then
    Str := str + '|DELTAMOVE';
  if (Win^.IDCMPFlags and IDCMP_VANILLAKEY) <> 0 then
    Str := str + '|VANILLAKEY';
  if (Win^.IDCMPFlags and IDCMP_IDCMPUPDATE) <> 0 then
    Str := str + '|IDCMPUPDATE';
  if (Win^.IDCMPFlags and IDCMP_MENUHELP) <> 0 then
    Str := str + '|MENUHELP';
  if (Win^.IDCMPFlags and IDCMP_CHANGEWINDOW) <> 0 then
    Str := str + '|CHANGEWINDOW';
  if (Win^.IDCMPFlags and IDCMP_GADGETHELP) <> 0 then
    Str := str + '|GADGETHELP';
  if Length(Str) > 0 then
    Delete(str, 1, 1);
  AddCell('IDCMP Flags', '($' + IntToHex(Win^.IDCMPFlags, 8) + ')' + str);
  AddCell('ReqCount', IntToStr(Win^.ReqCount));
  AddCell('Border', 'left:' + IntToStr(Win^.BorderLeft) + ' top:' + IntToStr(Win^.BorderTop) + ' right:' + IntToStr(Win^.BorderRight) + ' bottom:' + IntToStr(Win^.BorderBottom));
  AddCell('ScreenTitle', string(Win^.ScreenTitle));
  AddCell('GZZ Size', IntToStr(Win^.GZZWidth) + 'x' + IntToStr(Win^.GZZHeight));
  AddCell('Rel. Pos' , IntToStr(Win^.RelLeftEdge) + ',' + IntToStr(Win^.RelLeftEdge));
  AddCell('UserData', Win^.UserData);
  AddCell('MoreFlags', '$' + IntToHex(Win^.MoreFlags, 8));
end;

procedure TMainForm.TreeNodeChanged(Sender: TObject);
var
  Sel: TfpgTreeNode;
  Idx: Integer;
  NPtr: Pointer;
begin
  Sel := Tree.Selection;
  Grid1.BeginUpdate;
  Grid1.RowCount := 0;
  if Assigned(Sel) then
  begin
    Idx := Integer(Sel.Data);
    if (Idx >= 0) and (Idx < Length(ItemList)) then
    begin
      NPtr := StillExists(Idx);
      if Assigned(NPtr) then
      begin
        case ItemList[Idx].PtrType of
          ENTRYTYPE_SCREEN: begin
            InspectScreen(NPtr);
          end;
          ENTRYTYPE_WINDOW: begin
            InspectWindow(NPtr);
          end;
        end;
      end;
    end;
  end else
  begin
    Grid1.RowCount := 0;
  end;
  Grid1.EndUpdate;
end;

constructor TMainForm.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
end;

destructor TMainForm.Destroy;
begin
  if Assigned(IBase) then
    Closelibrary(plibrary(iBase));
  tree.ImageList := nil;
  tree.StateImageList := nil;
  inherited Destroy;
end;

procedure TMainForm.AfterCreate;
begin
  inherited AfterCreate;
  IBase :=pIntuitionBase(Openlibrary('intuition.library', 36));
  if not Assigned(IBase) then
  begin
    self.Close;
    Exit;
  end;
  {@VFD_BODY_BEGIN: MainForm}
  Name := 'MainForm';
  SetPosition(399, 184, 709, 333);
  WindowTitle := 'Show Windows';
  Hint := '';
  WindowPosition := wpScreenCenter;

  tree := TfpgTreeView.Create(self);
  with tree do
  begin
    Name := 'tree';
    SetPosition(8, 8, 308, 283);
    Anchors := [anLeft,anTop,anBottom];
    FontDesc := '#Label1';
    Hint := '';
    ScrollWheelDelta := 30;
    ShowImages := True;
    TabOrder := 0;
    OnChange  := @TreeNodeChanged;
  end;

  btnRefresh := TfpgButton.Create(self);
  with btnRefresh do
  begin
    Name := 'btnRefresh';
    SetPosition(203, 303, 108, 22);
    Anchors := [anLeft,anBottom];
    Text := 'Refresh Windows';
    FontDesc := '#Label1';
    Hint := '';
    ImageName := '';
    TabOrder := 3;
    OnClick := @btnRefreshClicked;
  end;

  btnCollapseAll := TfpgButton.Create(self);
  with btnCollapseAll do
  begin
    Name := 'btnCollapseAll';
    SetPosition(12, 303, 92, 22);
    Anchors := [anLeft,anBottom];
    Text := 'Collapse All';
    FontDesc := '#Label1';
    Hint := '';
    ImageName := '';
    TabOrder := 19;
    OnClick := @btnCollapseAllClicked;
  end;

  btnExpandAll := TfpgButton.Create(self);
  with btnExpandAll do
  begin
    Name := 'btnExpandAll';
    SetPosition(108, 303, 92, 22);
    Anchors := [anLeft,anBottom];
    Text := 'Expand All';
    FontDesc := '#Label1';
    Hint := '';
    ImageName := '';
    TabOrder := 20;
    OnClick := @btnExpandAllClicked;
  end;

  Grid1 := TfpgStringGrid.Create(self);
  with Grid1 do
  begin
    Name := 'Grid1';
    SetPosition(324, 7, 376, 280);
    Anchors := [anLeft,anRight,anTop,anBottom];
    BackgroundColor := TfpgColor($80000002);
    FontDesc := '#Grid';
    HeaderFontDesc := '#GridHeader';
    Hint := '';
    RowCount := 0;
    RowSelect := True;
    TabOrder := 5;
  end;

  btnMarkWindow := TfpgButton.Create(self);
  with btnMarkWindow do
  begin
    Name := 'btnMarkWindow';
    SetPosition(324, 303, 100, 22);
    Anchors := [anLeft,anBottom];
    Text := 'Show Selected';
    FontDesc := '#Label1';
    Hint := '';
    ImageName := '';
    TabOrder := 6;
    OnClick := @btnShowSelected;
  end;

  {@VFD_BODY_END: MainForm}
  Grid1.AddColumn('Name', 100);
  Grid1.AddColumn('Value', 500);

  RefreshWindowList;
end;

procedure TMainForm.btnCollapseAllClicked(Sender: TObject);
begin
  tree.FullCollapse;
end;

procedure TMainForm.btnExpandAllClicked(Sender: TObject);
begin
  tree.FullExpand;
end;

procedure TMainForm.MarkWindow(Window: PWindow);
var
  Scr: PScreen;
  Pen: LongInt;
  OPen: LongInt;
begin
  Scr := Window^.WScreen;
  if Assigned(Scr) then
  begin
    Pen := ObtainBestPenA(Scr^.ViewPort.ColorMap, LongWord(-1), 0, 0, nil);
    OPen := GetAPen(Window^.RPort);
    SetAPen(Window^.RPort, Pen);
    Move(Window^.RPort, 0, 0);
    if (Window^.Flags and WFLG_GIMMEZEROZERO) = 0 then
    begin
      Draw(Window^.RPort, Window^.Width - 1, 0);
      Draw(Window^.RPort, Window^.Width - 1, Window^.Height - 1);
      Draw(Window^.RPort, 0, Window^.Height - 1);
    end else
    begin
      Draw(Window^.RPort, Window^.GZZWidth - 1, 0);
      Draw(Window^.RPort, Window^.GZZWidth - 1, Window^.GZZHeight - 1);
      Draw(Window^.RPort, 0, Window^.GZZHeight - 1);
    end;
    Draw(Window^.RPort, 0, 0);
    SetAPen(Window^.RPort, OPen);
    ReleasePen(Scr^.ViewPort.ColorMap, Pen);
  end;
end;

procedure TMainForm.btnShowSelected(Sender: TObject);
var
  Sel: TfpgTreeNode;
  Idx: Integer;
  NPtr: Pointer;
  FrontWin: PWindow;
begin
  Sel := Tree.Selection;
  if Assigned(Sel) then
  begin
    Idx := Integer(Sel.Data);
    if (Idx >= 0) and (Idx < Length(ItemList)) then
    begin
      NPtr := StillExists(Idx);
      if Assigned(NPtr) then
      begin
        case ItemList[Idx].PtrType of
          ENTRYTYPE_SCREEN: begin
            //
          end;
          ENTRYTYPE_WINDOW: begin
            FrontWin := PWindow(NPtr);
            while Assigned(FrontWin^.Parent2) do
            begin
              FrontWin := FrontWin^.Parent2;
            end;
            if Assigned(FrontWin^.WScreen) then
              ScreenToFront(PScreen(FrontWin^.WScreen));
            WindowToFront(FrontWin);
            MarkWindow(NPtr);
          end;
        end;
      end;
    end;
  end;
end;


procedure MainProc;
var
  frm: TMainForm;
begin
  fpgApplication.Initialize;
  frm := TMainForm.Create(nil);
  try
    frm.Show;
    fpgApplication.Run;
  finally
    frm.Free;
  end;
end;

begin
  MainProc;
end.


